from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker
from typing import Generator


class PostgresConnection:
    def __init__(self, postgres_user, postgres_password, postgres_host, postgres_port, database_name, read_only=True):
        self.POSTGRES_USER = postgres_user
        self.POSTGRES_PASSWORD = postgres_password
        self.POSTGRES_HOST = postgres_host
        self.POSTGRES_PORT = postgres_port
        self.DATABASE_NAME = database_name
        self.__create_session(read_only)

    def __make_connection_url(self):
        connection_url = f"postgresql://{self.POSTGRES_USER}:{self.POSTGRES_PASSWORD}@{self.POSTGRES_HOST}:{self.POSTGRES_PORT}/{self.DATABASE_NAME}"

        return connection_url

    def __create_session(self, read_only):
        connection_url = self.__make_connection_url()
        engine = create_engine(url=connection_url, future=True, echo=False)
        # Create a local session maker to interact with the db via ORM
        self.SessionLocal = sessionmaker(autocommit=False, autoflush=False, bind=engine)
        if read_only:
            self.SessionLocal.flush = self.__abort_ro  # now it won't flush!

    def __abort_ro(*args, **kwargs):
        return

    def get_db(self) -> Generator:
        db = self.SessionLocal()
        try:
            yield db
        finally:
            db.close()
